/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*-
 *
 * nautilus-burn-drive.h: easy to use cd burner software
 *
 * Copyright (C) 2002-2004 Red Hat, Inc.
 * Copyright (C) 2005 William Jon McCann <mccann@jhu.edu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Alexander Larsson <alexl@redhat.com>
 *          Bastien Nocera <hadess@hadess.net>
 *          William Jon McCann <mccann@jhu.edu>
 */

#ifndef OGMDVD_DRIVE_H
#define OGMDVD_DRIVE_H

#include <glib-object.h>

G_BEGIN_DECLS

#define OGMDVD_TYPE_DRIVE            (ogmdvd_drive_get_type ())
#define OGMDVD_DRIVE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj),  OGMDVD_TYPE_DRIVE, OGMDvdDrive))
#define OGMDVD_DRIVE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass),   OGMDVD_TYPE_DRIVE, OGMDvdDriveClass))
#define OGMDVD_IS_DRIVE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj),  OGMDVD_TYPE_DRIVE))
#define OGMDVD_IS_DRIVE_CLASS(klass) (G_TYPE_INSTANCE_GET_CLASS ((klass), OGMDVD_TYPE_DRIVE))

typedef enum 
{
  OGMDVD_MEDIA_TYPE_BUSY,
  OGMDVD_MEDIA_TYPE_ERROR,
  OGMDVD_MEDIA_TYPE_UNKNOWN,
  OGMDVD_MEDIA_TYPE_CD,
  OGMDVD_MEDIA_TYPE_CDR,
  OGMDVD_MEDIA_TYPE_CDRW,
  OGMDVD_MEDIA_TYPE_DVD,
  OGMDVD_MEDIA_TYPE_DVDR,
  OGMDVD_MEDIA_TYPE_DVDRW,
  OGMDVD_MEDIA_TYPE_DVD_RAM,
  OGMDVD_MEDIA_TYPE_DVD_PLUS_R,
  OGMDVD_MEDIA_TYPE_DVD_PLUS_RW,
  OGMDVD_MEDIA_TYPE_DVD_PLUS_R_DL
} OGMDvdMediaType;

typedef enum
{
  OGMDVD_DRIVE_TYPE_FILE                   = 1 << 0,
  OGMDVD_DRIVE_TYPE_CD_RECORDER            = 1 << 1,
  OGMDVD_DRIVE_TYPE_CDRW_RECORDER          = 1 << 2,
  OGMDVD_DRIVE_TYPE_DVD_RAM_RECORDER       = 1 << 3,
  /* Drives are usually DVD-R and DVD-RW */
  OGMDVD_DRIVE_TYPE_DVD_RW_RECORDER        = 1 << 4,
  OGMDVD_DRIVE_TYPE_DVD_PLUS_R_RECORDER    = 1 << 5,
  OGMDVD_DRIVE_TYPE_DVD_PLUS_RW_RECORDER   = 1 << 6,
  OGMDVD_DRIVE_TYPE_CD_DRIVE               = 1 << 7,
  OGMDVD_DRIVE_TYPE_DVD_DRIVE              = 1 << 8,
  OGMDVD_DRIVE_TYPE_DVD_PLUS_R_DL_RECORDER = 1 << 9
} OGMDvdDriveType;

typedef struct _OGMDvdDrive      OGMDvdDrive;
typedef struct _OGMDvdDriveClass OGMDvdDriveClass;
typedef struct _OGMDvdDrivePriv  OGMDvdDrivePriv;

struct _OGMDvdDrive
{
  GObject parent_instance;

  OGMDvdDrivePriv *priv;
};


struct _OGMDvdDriveClass
{
  GObjectClass parent_class;

  void (*media_added)   (OGMDvdDrive *drive);
  void (*media_removed) (OGMDvdDrive *drive);
};

GType           ogmdvd_drive_get_type            (void);
GList *         ogmdvd_drive_get_list            (void);

OGMDvdDrive *   ogmdvd_drive_new                 (void);
OGMDvdDrive *   ogmdvd_drive_new_from_path       (const gchar *device);

gint            ogmdvd_drive_get_drive_type      (OGMDvdDrive *drive);
gint            ogmdvd_drive_get_media_type      (OGMDvdDrive *drive);
gchar *         ogmdvd_drive_get_display_name    (OGMDvdDrive *drive);
const gchar *   ogmdvd_drive_get_device          (OGMDvdDrive *drive);

gboolean        ogmdvd_drive_eject               (OGMDvdDrive *drive);
gboolean        ogmdvd_drive_door_is_open        (OGMDvdDrive *drive);

void            ogmdvd_drive_set_monitor_enabled (OGMDvdDrive *drive, 
                                                  gboolean    enabled);
gboolean        ogmdvd_drive_get_monitor_enabled (OGMDvdDrive *drive);

G_END_DECLS

#endif
